@@ -41,7 +41,25 @@ public:
     {
         return true;
     }
+
+    /**
+     * Tries to obtain a launchable desktop file for this snap, in this order:
+     *
+     * 1. Any app with the same name as the snap and a desktop file (the main app)
+     * 2. The first app with a desktop file (the next best app)
+     * 3. The expected desktop file for the main app (<snap_name>_<snap_name>.desktop)
+     *
+     * @return The fileName of the selected launchable desktop file.
+     */
+    QString launchableDesktopFile() const;
+
+    /**
+     * Launches a snap using its desktop file.
+     *
+     * If no desktop file is found, defaults back to `snap run`, which will fail in Ubuntu Core environments.
+     */
     void invokeApplication() const override;
+
     void fetchChangelog() override;
     void fetchScreenshots() override;
     QString author() const override;
@@ -65,7 +83,7 @@ public:
 
     QStringList topObjects() const override;
 
-    QString channel() const;
+    QString channel();
     void setChannel(const QString &channel);
 
     quint64 installedSize() const;
@@ -90,6 +108,7 @@ public:
     quint64 m_downloadSize;
 
     QSharedPointer<QSnapdSnap> m_snap;
+    QString m_channel;
     mutable QVariant m_icon;
     static const QStringList s_topObjects;
 };
