@@ -22,6 +22,8 @@
 #include <KIO/CommandLauncherJob>
 
 #include "../libdiscover/utils.h"
+#include "../libdiscover/UpdateModel/RefreshNotifierDBus.h"
+#include "Login1ManagerInterface.h"
 #include "updatessettings.h"
 #include <chrono>
 
@@ -44,6 +46,7 @@ DiscoverNotifier::DiscoverNotifier(QObject *parent)
     if (auto info = QNetworkInformation::instance()) {
         connect(info, &QNetworkInformation::reachabilityChanged, this, &DiscoverNotifier::stateChanged);
         connect(info, &QNetworkInformation::transportMediumChanged, this, &DiscoverNotifier::stateChanged);
+        connect(info, &QNetworkInformation::isBehindCaptivePortalChanged, this, &DiscoverNotifier::stateChanged);
     } else {
         qWarning() << "QNetworkInformation has no backend. Is NetworkManager.service running?";
     }
@@ -81,7 +84,26 @@ DiscoverNotifier::DiscoverNotifier(QObject *parent)
     updateStatusNotifier();
 
     // Only fetch updates after the system is comfortably booted
-    QTimer::singleShot(0s, this, &DiscoverNotifier::recheckSystemUpdateNeeded);
+    QTimer::singleShot(20s, this, &DiscoverNotifier::recheckSystemUpdateNeeded);
+
+    auto login1 = new OrgFreedesktopLogin1ManagerInterface(QStringLiteral("org.freedesktop.login1"),
+                                                           QStringLiteral("/org/freedesktop/login1"),
+                                                           QDBusConnection::systemBus(),
+                                                           this);
+    connect(login1, &OrgFreedesktopLogin1ManagerInterface::PrepareForSleep, this, [this](bool sleeping) {
+        // if we just woke up and have not checked in the notification interval
+        if (!sleeping && m_lastUpdate.secsTo(QDateTime::currentDateTimeUtc()) > m_settings->requiredNotificationInterval()) {
+            QTimer::singleShot(20s, this, &DiscoverNotifier::recheckSystemUpdateNeeded);
+        }
+    });
+
+    // Listen to broadcasts from discover about notification changes.
+    QDBusConnection::sessionBus().connect(QString(),
+                                          RefreshNotifierDBus::path,
+                                          RefreshNotifierDBus::interface,
+                                          RefreshNotifierDBus::notifyNotifier,
+                                          this,
+                                          SLOT(recheckSystemUpdateNeeded()));
 }
 
 DiscoverNotifier::~DiscoverNotifier() = default;
@@ -193,6 +215,9 @@ static bool isConnectionAdequate()
     if (!info) {
         return false; // no backend available (e.g. NetworkManager not running), assume not adequate
     }
+    if (info->supports(QNetworkInformation::Feature::CaptivePortal) && info->isBehindCaptivePortal()) {
+        return false;
+    }
     if (info->supports(QNetworkInformation::Feature::Metered)) {
         return !info->isMetered();
     } else {
@@ -247,7 +272,7 @@ QString DiscoverNotifier::iconName() const
     case NoUpdates:
         return QStringLiteral("update-none");
     case RebootRequired:
-        return QStringLiteral("system-reboot");
+        return QStringLiteral("system-reboot-update");
     case Offline:
         return QStringLiteral("offline");
     case Busy:
@@ -277,6 +302,7 @@ QString DiscoverNotifier::message() const
 
 void DiscoverNotifier::recheckSystemUpdateNeeded()
 {
+    m_lastUpdate = QDateTime::currentDateTimeUtc();
     for (BackendNotifierModule *module : std::as_const(m_backends))
         module->recheckSystemUpdateNeeded();
 
@@ -300,13 +326,15 @@ void DiscoverNotifier::showRebootNotification()
                                                        KNotification::Persistent | KNotification::DefaultEvent,
                                                        QStringLiteral("discoverabstractnotifier"));
 
-    auto restartAction = notification->addAction(i18nc("@action:button", "Restart"));
-    connect(restartAction, &KNotificationAction::activated, this, &DiscoverNotifier::reboot);
+    auto restartAction = notification->addAction(i18nc("@action:button", "Update and Restart"));
+    auto shutdownAction = notification->addAction(i18nc("@action:button", "Update and Shut Down"));
+    connect(restartAction, &KNotificationAction::activated, this, &DiscoverNotifier::rebootPrompt);
+    connect(shutdownAction, &KNotificationAction::activated, this, &DiscoverNotifier::shutdownPrompt);
 
     notification->sendEvent();
 }
 
-void DiscoverNotifier::reboot()
+void DiscoverNotifier::rebootPrompt()
 {
     auto method = QDBusMessage::createMethodCall(QStringLiteral("org.kde.LogoutPrompt"),
                                                  QStringLiteral("/LogoutPrompt"),
@@ -315,6 +343,24 @@ void DiscoverNotifier::reboot()
     QDBusConnection::sessionBus().asyncCall(method);
 }
 
+void DiscoverNotifier::shutdownPrompt()
+{
+    auto method = QDBusMessage::createMethodCall(QStringLiteral("org.kde.LogoutPrompt"),
+                                                 QStringLiteral("/LogoutPrompt"),
+                                                 QStringLiteral("org.kde.LogoutPrompt"),
+                                                 QStringLiteral("promptShutDown"));
+    QDBusConnection::sessionBus().asyncCall(method);
+}
+
+void DiscoverNotifier::promptAll()
+{
+    auto method = QDBusMessage::createMethodCall(QStringLiteral("org.kde.LogoutPrompt"),
+                                                 QStringLiteral("/LogoutPrompt"),
+                                                 QStringLiteral("org.kde.LogoutPrompt"),
+                                                 QStringLiteral("promptAll"));
+    QDBusConnection::sessionBus().asyncCall(method);
+}
+
 void DiscoverNotifier::foundUpgradeAction(UpgradeAction *action)
 {
     updateStatusNotifier();
@@ -351,4 +397,10 @@ void DiscoverNotifier::setBusy(bool isBusy)
     Q_EMIT stateChanged();
 }
 
+void DiscoverNotifier::recheckSystemUpdateNeededAndNotifyApp()
+{
+    recheckSystemUpdateNeeded();
+    RefreshNotifierDBus::notify(RefreshNotifierDBus::notifyApp);
+}
+
 #include "moc_DiscoverNotifier.cpp"
