@@ -24,7 +24,9 @@ UnattendedUpdates::UnattendedUpdates(DiscoverNotifier *parent)
 
 UnattendedUpdates::~UnattendedUpdates() noexcept
 {
-    KIdleTime::instance()->removeAllIdleTimeouts();
+    if (m_idleTimeoutId.has_value()) {
+        KIdleTime::instance()->removeIdleTimeout(m_idleTimeoutId.value());
+    }
 }
 
 void UnattendedUpdates::checkNewState()
@@ -33,30 +35,33 @@ void UnattendedUpdates::checkNewState()
     DiscoverNotifier *notifier = static_cast<DiscoverNotifier *>(parent());
 
     // Only allow offline updating every 3h. It should keep some peace to our users, especially on rolling distros
-    const QDateTime updateableTime = notifier->settings()->lastUnattendedTrigger().addSecs((3h).count());
+    const QDateTime updateableTime = notifier->settings()->lastUnattendedTrigger().addSecs(std::chrono::seconds(3h).count());
     if (updateableTime > QDateTime::currentDateTimeUtc()) {
         qDebug() << "skipping update, already updated on" << notifier->settings()->lastUnattendedTrigger().toString();
         return;
     }
 
-    if (!KIdleTime::instance()->idleTimeouts().isEmpty()) {
-        qDebug() << "already waiting for an idle time";
-        return;
-    }
-
-    if (notifier->hasUpdates()) {
+    const bool doTrigger = notifier->hasUpdates() && !notifier->isBusy();
+    if (doTrigger && !m_idleTimeoutId.has_value()) {
         qDebug() << "waiting for an idle moment";
         // If the system is untouched for 15 minutes, trigger the unattened update
-        KIdleTime::instance()->addIdleTimeout(int(std::chrono::milliseconds(15min).count()));
-    } else {
+        m_idleTimeoutId = KIdleTime::instance()->addIdleTimeout(int(std::chrono::milliseconds(15min).count()));
+    } else if (!doTrigger && m_idleTimeoutId.has_value()) {
         qDebug() << "nothing to do";
-        KIdleTime::instance()->removeAllIdleTimeouts();
+        KIdleTime::instance()->removeIdleTimeout(m_idleTimeoutId.value());
+        m_idleTimeoutId.reset();
     }
 }
 
-void UnattendedUpdates::triggerUpdate()
+void UnattendedUpdates::triggerUpdate(int timeoutId)
 {
-    KIdleTime::instance()->removeAllIdleTimeouts();
+    if (!m_idleTimeoutId.has_value() || timeoutId != m_idleTimeoutId.value()) {
+        return;
+    }
+
+    KIdleTime::instance()->removeIdleTimeout(m_idleTimeoutId.value());
+    m_idleTimeoutId.reset();
+
     DiscoverNotifier *notifier = static_cast<DiscoverNotifier *>(parent());
     if (!notifier->hasUpdates() || notifier->isBusy()) {
         return;
@@ -69,12 +74,15 @@ void UnattendedUpdates::triggerUpdate()
     connect(process, QOverload<int, QProcess::ExitStatus>::of(&QProcess::finished), this, [this, process](int exitCode, QProcess::ExitStatus exitStatus) {
         qDebug() << "Finished running plasma-discover-update" << exitCode << exitStatus;
         DiscoverNotifier *notifier = static_cast<DiscoverNotifier *>(parent());
-        notifier->setBusy(false);
         process->deleteLater();
         notifier->settings()->setLastUnattendedTrigger(QDateTime::currentDateTimeUtc());
+        notifier->settings()->save();
+        notifier->setBusy(false);
     });
 
     notifier->setBusy(true);
     process->start(QStringLiteral("plasma-discover-update"), {QStringLiteral("--offline")});
     qInfo() << "started unattended update" << QDateTime::currentDateTimeUtc();
 }
+
+#include "moc_UnattendedUpdates.cpp"
